@@ -1,5 +1,5 @@
 /* Tarlz - Archiver with multimember lzip compression
-   Copyright (C) 2013-2021 Antonio Diaz Diaz.
+   Copyright (C) 2013-2022 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -19,31 +19,26 @@
 
 #include <cctype>
 #include <cerrno>
-#include <climits>
 #include <cstdlib>
-#include <cstring>
-#include <string>
-#include <vector>
 #include <pthread.h>
-#include <stdint.h>
 #include <unistd.h>
 
-#include "arg_parser.h"
 #include "tarlz.h"
+#include "arg_parser.h"
 
 
 void xinit_mutex( pthread_mutex_t * const mutex )
   {
   const int errcode = pthread_mutex_init( mutex, 0 );
   if( errcode )
-    { show_error( "pthread_mutex_init", errcode ); cleanup_and_fail(); }
+    { show_error( "pthread_mutex_init", errcode ); exit_fail_mt(); }
   }
 
 void xinit_cond( pthread_cond_t * const cond )
   {
   const int errcode = pthread_cond_init( cond, 0 );
   if( errcode )
-    { show_error( "pthread_cond_init", errcode ); cleanup_and_fail(); }
+    { show_error( "pthread_cond_init", errcode ); exit_fail_mt(); }
   }
 
 
@@ -51,14 +46,14 @@ void xdestroy_mutex( pthread_mutex_t * const mutex )
   {
   const int errcode = pthread_mutex_destroy( mutex );
   if( errcode )
-    { show_error( "pthread_mutex_destroy", errcode ); cleanup_and_fail(); }
+    { show_error( "pthread_mutex_destroy", errcode ); exit_fail_mt(); }
   }
 
 void xdestroy_cond( pthread_cond_t * const cond )
   {
   const int errcode = pthread_cond_destroy( cond );
   if( errcode )
-    { show_error( "pthread_cond_destroy", errcode ); cleanup_and_fail(); }
+    { show_error( "pthread_cond_destroy", errcode ); exit_fail_mt(); }
   }
 
 
@@ -66,7 +61,7 @@ void xlock( pthread_mutex_t * const mutex )
   {
   const int errcode = pthread_mutex_lock( mutex );
   if( errcode )
-    { show_error( "pthread_mutex_lock", errcode ); cleanup_and_fail(); }
+    { show_error( "pthread_mutex_lock", errcode ); exit_fail_mt(); }
   }
 
 
@@ -74,7 +69,7 @@ void xunlock( pthread_mutex_t * const mutex )
   {
   const int errcode = pthread_mutex_unlock( mutex );
   if( errcode )
-    { show_error( "pthread_mutex_unlock", errcode ); cleanup_and_fail(); }
+    { show_error( "pthread_mutex_unlock", errcode ); exit_fail_mt(); }
   }
 
 
@@ -82,7 +77,7 @@ void xwait( pthread_cond_t * const cond, pthread_mutex_t * const mutex )
   {
   const int errcode = pthread_cond_wait( cond, mutex );
   if( errcode )
-    { show_error( "pthread_cond_wait", errcode ); cleanup_and_fail(); }
+    { show_error( "pthread_cond_wait", errcode ); exit_fail_mt(); }
   }
 
 
@@ -90,7 +85,7 @@ void xsignal( pthread_cond_t * const cond )
   {
   const int errcode = pthread_cond_signal( cond );
   if( errcode )
-    { show_error( "pthread_cond_signal", errcode ); cleanup_and_fail(); }
+    { show_error( "pthread_cond_signal", errcode ); exit_fail_mt(); }
   }
 
 
@@ -98,7 +93,7 @@ void xbroadcast( pthread_cond_t * const cond )
   {
   const int errcode = pthread_cond_broadcast( cond );
   if( errcode )
-    { show_error( "pthread_cond_broadcast", errcode ); cleanup_and_fail(); }
+    { show_error( "pthread_cond_broadcast", errcode ); exit_fail_mt(); }
   }
 
 
@@ -113,8 +108,8 @@ unsigned long long parse_octal( const uint8_t * const ptr, const int size )
   }
 
 
-/* Returns the number of bytes really read.
-   If (returned value < size) and (errno == 0), means EOF was reached.
+/* Return the number of bytes really read.
+   If (value returned < size) and (errno == 0), means EOF was reached.
 */
 int readblock( const int fd, uint8_t * const buf, const int size )
   {
@@ -132,8 +127,8 @@ int readblock( const int fd, uint8_t * const buf, const int size )
   }
 
 
-/* Returns the number of bytes really written.
-   If (returned value < size), it is always an error.
+/* Return the number of bytes really written.
+   If (value returned < size), it is always an error.
 */
 int writeblock( const int fd, const uint8_t * const buf, const int size )
   {
